// $Id: tagmap.js,v 1.1.2.5 2010/08/10 11:33:22 kartelusm Exp $

/**
 * @file Tagmap integration in Drupal.
 */

/*
 * global $, Drupal, G_NORMAL_MAP, G_SATELLITE_MAP, G_HYBRID_MAP,
 * G_PHYSICAL_MAP,
 */
/*
 * global GMap2, GLatLng, GSmallZoomControl, GSmallMapControl, GLargeMapControl,
 * GMapTypeControl,
 */
/*
 * global GHierarchicalMapTypeControl, GMenuMapTypeControl, GLatLngBounds,
 * GIcon,
 */

Drupal.tagmap = new function() {
	var gmarkers = new Array();

	this.getMarker = function(mapnum_id, id) {
		return gmarkers[mapnum_id][id];
	};

	this.numberOfMarkers = function(mapnum_id) {
		return gmarkers[mapnum_id].length;
	};

	this.handleSelected = function(mapnum_id, opt) {
		var indexNo = opt[opt.selectedIndex].index;
		GEvent.trigger(gmarkers[mapnum_id][indexNo], "click");

	};
	
	this.addBonusLayers = function(mapid, map, mapTypes) {
		var default_set = false;
	  	var layers = Drupal.settings.tagmap[mapid].layer;
	  	var defaultlayer = Drupal.settings.tagmap[mapid].defaultlayer;
	  	if(layers != undefined && layers != null){
		  	for ( var i = 0; i <layers.length; i++) {
		  	    var custommap = GMapBox(layers[i].name, layers[i].title, layers[i].options);
		  	    map.addMapType(custommap);
		  	    if(defaultlayer == layers[i].name && !default_set){
		  	      map.setMapType(GMapBox(layers[i].name, layers[i].title, layers[i].options));
		  	      default_set = true;
		  	    }
		  	    
		  	}
	  	}
	  	var overlays = Drupal.settings.tagmap[mapid].overlay;
	  	if(overlays != undefined && overlays != null){
		  	for ( var i = 0; i <overlays.length; i++) {
		  	    var customoverlay  = GMapBox(overlays[i].name, overlays[i].title, overlays[i].options);
		  	    map.addOverlay(customoverlay);
		  	}
	  	}
	  	if (!default_set)
	  		map.setMapType(mapTypes[defaultlayer]);
	};

	/**
	 * Dropdown list with the tags as an option. On click, go to the tag on the
	 * map and open the info
	 */
	this.renderOptions = function(mapid, mapnum_id, id) {
		if (Drupal.settings.tagmap[mapid].clustering != 'clusterer2'
				&& Drupal.settings.tagmap[mapid].clustering != 'markerclusterer_packed'
				&& !$('#' + id).parent().hasClass('dropdown-processed')) {
			$('#' + id).parent().append(
					"<div id=\"tagmap-select-div-" + mapid
							+ "\" class=\"tagmap-select-div\"></div>")
			$("#tagmap-select-div-" + mapid)
					.append(
							"<select onchange=\"Drupal.tagmap.handleSelected("
									+ mapnum_id
									+ ", this);\" id=\"tagmap-select-select-options-"
									+ mapid
									+ "\" class=\"tagmap-select-select-options\"></select>")
			var len = Drupal.tagmap.numberOfMarkers(mapnum_id);
			for ( var i = 0; i < len; i++) {
				$("#tagmap-select-select-options-" + mapid).append(
						"<option  value=\"" + i + "\">"
								+ Drupal.tagmap.getMarker(mapnum_id, i).text_
								+ "</option>")
			}
			$('#' + id).parent().addClass('dropdown-processed');
		}
	};

	/**
	 * Encapsulation of marker creation. Needed so that events trigger correctly
	 */
	this.createMarker = function(lat, lon, opts, text) {
		var marker = new TextMarker(new GLatLng(lat, lon), opts);
		GEvent.addListener(marker, "click", function() {
			marker.openInfoWindowHtml(text);
		});
		return marker;
	};

	this.init = function() {
		var obj = this;
		if (GBrowserIsCompatible()) {
			var mapid = obj.id.split('-');
			mapid.pop();
			mapid.shift();
			var tmp = new String(mapid);
			var ind = tmp.indexOf("map");
			var mapnum_id = tmp.substr(4, ind - 4);
			mapid = mapid.join('-');
			obj.vars = Drupal.settings.tagmap[mapid];
			obj.vars.initial_state.zoom = parseInt(obj.vars.initial_state.zoom);
			var mapTypes = {
				'map' : G_NORMAL_MAP,
				'satellite' : G_SATELLITE_MAP,
				'hybrid' : G_HYBRID_MAP,
				'physical' : G_PHYSICAL_MAP
			};
			obj.vars.mapTypes = [];
			for (layer in obj.vars.baselayers) {
				obj.vars.mapTypes.push(mapTypes[layer]);
			}	
			var map = new GMap2(document.getElementById(obj.id), obj.vars);
			// Setting the map center. Check if user provided the default center
			// coordinates,
			// otherwise map center will be calculated on the basis of the
			// markers position on the map.
			var center = new GLatLng(0, 0);
			if (obj.vars.initial_state.center) {
				center = obj.vars.initial_state.center.split(',');
				center = new GLatLng(center[0], center[1]);
			}
			map.setCenter(center);

			
			
			// Setting map control type
			if (obj.vars.controltype === 'Micro') {
				map.addControl(new GSmallZoomControl());
			} else if (obj.vars.controltype === 'Small') {
				map.addControl(new GSmallMapControl());
			} else if (obj.vars.controltype === 'Large') {
				map.addControl(new GLargeMapControl());
			}
			// Setting map menu type control
			if (obj.vars.mtc === 'standard') {
				map.addControl(new GMapTypeControl());
			} else if (obj.vars.mtc === 'hier') {
				map.addControl(new GHierarchicalMapTypeControl());
			} else if (obj.vars.mtc === 'menu') {
				map.addControl(new GMenuMapTypeControl());
			}
			// Setting scrollwheel and continuous zoom
			if (!obj.vars.behavior.nomousezoom) {
				map.enableScrollWheelZoom();
				map.enableContinuousZoom();
			}

			//map.setMapType(mapTypes[obj.vars.maptype]);
			Drupal.tagmap.addBonusLayers(mapid, map, mapTypes);
			
			// Generation of Text markers
			var cluster = [];
			var bounds = new GLatLngBounds;
			gmarkers[mapnum_id] = new Array();
			if (obj.vars.clustering == 'clusterer2') {
				var clusterer = new Clusterer(map);
				clusterer.SetMaxVisibleMarkers(Drupal.settings.tagmap[mapid].maxVisibleMarkers);
				clusterer.SetMinMarkersPerCluster(Drupal.settings.tagmap[mapid].minMarkersPerCluster);
				clusterer.SetMaxLinesPerInfoBox(Drupal.settings.tagmap[mapid].maxLinesPerBox);
			}

			for ( var i = 0; i < obj.vars.markers.length; i++) {
				var opts = {};
				opts.text = obj.vars.markers[i].term;
				opts.weight = obj.vars.markers[i].weight;
				opts.icon = new GIcon(G_DEFAULT_ICON);
				opts.infoText = obj.vars.markers[i].infoText;
				opts.minFontSize = obj.vars.markers[i].minFontSize;
				opts.fontStyle = obj.vars.markers[i].fontStyle;
				opts.markerColor = obj.vars.markers[i].markerColor;
				var marker = Drupal.tagmap.createMarker(
						obj.vars.markers[i].latitude,
						obj.vars.markers[i].longitude, opts,
						obj.vars.markers[i].infoText);
				gmarkers[mapnum_id].push(marker);
				if (obj.vars.clustering == 'markerclusterer_packed') {
					cluster.push(marker);
				} else if (obj.vars.clustering == 'clusterer2') {
					clusterer.AddMarker(marker, obj.vars.markers[i].term);
				} else {
					map.addOverlay(marker);
				}

				bounds.extend(marker.getPoint());
			}
			if (obj.vars.clustering == 'markerclusterer_packed') {
				MarkerClusterer(map, cluster);
			}

			// If the default zoom level isn't provided, calculate the zoom
			// level based on
			// the markers position on the map, otherwise use the provided
			// value.
			if (obj.vars.initial_state.zoom == -1) {
				map.setZoom(map.getBoundsZoomLevel(bounds));
			} else {
				map.setZoom(obj.vars.initial_state.zoom);
			}

			// If the default map center coordinates aren't provided set
			// the map center based on the markers position on the map.
			if (!obj.vars.initial_state.center) {
				map.setCenter(bounds.getCenter());
			}
			Drupal.tagmap.renderOptions(mapid, mapnum_id, this.id);
		}
	};

	this.embed = function(id, rel, overwrite) {
		if (GBrowserIsCompatible()) {
			var mapid = rel.split('-');
			mapid.pop();
			mapid.shift();
			var width = Drupal.settings.tagmap[mapid].width;
			var height = Drupal.settings.tagmap[mapid].height;
			var scripts = document.getElementsByTagName('script');
			var scrString = "";
			for ( var i = 0; i < scripts.length; i++) {
				var tmp = scripts.item(i).text;
				if (tmp.search('tagmap') != -1 && tmp.search('infoText') != -1)
					scrString = tmp;

			}
			var path = Drupal.settings.basePath + 'tagmap/embed';
			$.ajax( {
				type : "POST",
				url : path,
				data : "id=" + id + "&rel=" + rel + "&clustering="
						+ Drupal.settings.tagmap[mapid].clustering
						+ "&settings=" + scrString+"&width="+width
						+"&height="+height+"&storage="+Drupal.settings.tagmap[mapid].storage+"&title="+Drupal.settings.tagmap[mapid].tagmap_title
						+"&vidiLink="+Drupal.settings.tagmap[mapid].vidi_link+"&viewLink="+Drupal.settings.tagmap[mapid].view_link+"&overwrite="+overwrite,
				success : function(msg) {
					var divid = $('#' + id).parent().attr('id');
					$('#' + divid).hide('fast');
					$('#' + divid).html(msg);
					$('#' + divid).fadeIn('slow');
				},
				error : function(msg) {
					alert('greska');
				}
			});
		}
	};
};

Drupal.behaviors.tagmap = function(context) {
	$('.tagmap-control:not(.tagmap-processed)', context).addClass(
			'tagmap-processed').each(Drupal.tagmap.init);
	$('.tagmap-embed-link', context).click(function() {
		Drupal.tagmap.embed($(this).attr('id'), $(this).attr('rel'), $(this).attr('overwrite'))
	});
}